"""Script for running a dam break simulation of UQ's dam break tank.


Ole Nielsen and Duncan Gray, GA - 2006 
"""


#----------------------------------------------------------------------------
# Import necessary modules
#----------------------------------------------------------------------------

# Standard modules
import time
import sys
from shutil import copy
from os import path

# Related major packages
from anuga.shallow_water import Domain, Reflective_boundary, \
                            Dirichlet_boundary, Time_boundary, File_boundary
from anuga.abstract_2d_finite_volumes.region import Set_region 
from anuga.fit_interpolate.interpolate import interpolate_sww2csv, \
     file_function
from anuga.abstract_2d_finite_volumes.util import start_screen_catcher, \
     copy_code_files
from anuga.shallow_water.data_manager import csv2dict
from numerical_tools import  err   # norm, corr,  

# Application specific imports
import create_mesh
import project
def main():
     
    slopes = [0]
    frictions = [0.01] #can enter multiple frictions eg [0.00, 0.01]
    inital_depths = [0.2] #can enter multiple initial depths eg [0.1, 0.2]
    gate_positions = [0.75]
   
    scenario(0, 0.01, 0.02, 0.75)
            

def scenario(slope, friction, inital_depth, gate_position):

    #-------------------------------------------------------------------------
    # Create the triangular mesh
    #-------------------------------------------------------------------------

    create_mesh.generate(project.mesh_filename,
                         gate_position,
                         is_course=True) # this creates the mesh
                         #is_course=False) # this creates the mesh

    head,tail = path.split(project.mesh_filename)
    #-------------------------------------------------------------------------
    # Setup computational domain
    #-------------------------------------------------------------------------
    domain = Domain(project.mesh_filename, use_cache = False, verbose = True)
   

    print 'Number of triangles = ', len(domain)
    print 'The extent is ', domain.get_extent()
    print domain.statistics()

    
    domain.set_name(project.basename)
    domain.set_datadir('.')
    domain.set_quantities_to_be_stored(['stage', 'xmomentum', 'ymomentum'])
    domain.set_minimum_storable_height(0.001)
    domain.set_store_vertices_uniquely(True)  # for writting to sww

    #-------------------------------------------------------------------------
    # Setup initial conditions
    #-------------------------------------------------------------------------


    def elevation_tilt(x, y):
        return x*slope
        
    domain.set_quantity('stage', elevation_tilt)
    domain.set_quantity('friction', friction) 
    domain.set_quantity('elevation',elevation_tilt)

    print 'Available boundary tags', domain.get_boundary_tags()
    domain.set_region('dam','stage',inital_depth,
                                 location = 'unique vertices') 

    Br = Reflective_boundary(domain)
    Bd = Dirichlet_boundary([0,0,0])  # to drain the water out.
    domain.set_boundary( {'wall': Br, 'edge': Bd} )

    #-------------------------------------------------------------------------
    # Evolve system through time
    #-------------------------------------------------------------------------
    t0 = time.time()

    for t in domain.evolve(yieldstep = 0.1, finaltime = 10): #enter timestep and final time
        domain.write_time()
    
        print 'That took %.2f seconds' %(time.time()-t0)
        print 'finished'

    # Load actual experimental results
    actual,title_index_dic = csv2dict(project.actual_filename)
    
    gauge_locations = [[0.4,0.2]]
    quantities = ['stage', 'elevation']
    file_instance = file_function( project.basename +".sww",
				      quantities = quantities,
				      interpolation_points = gauge_locations,
				      verbose = True,
				      use_cache = False)
    # create a list of the simulated_depths at the actual data times. 
    simulated_depths = []
    for atime in actual['time']:
        quantities_slice = file_instance(float(atime),
                                         point_id=0)
        depth = quantities_slice[0] - quantities_slice[1]
        simulated_depths.append(depth)
    flume_depths = actual["0.4:0.2"]
    flume_depths = [float(i) for i in flume_depths]
    # calc the norm
    #print "map(None, simulated_depths, flume_depths)", \
    #      map(None, simulated_depths, flume_depths)
    normmax = 0
    norm = err(simulated_depths,
               flume_depths, 2, relative = True)  # 2nd norm (rel. RMS
    if norm > normmax:
        normmax = norm
    print norm
    print "norm", norm
#-------------------------------------------------------------
if __name__ == "__main__":
    main()
    
